==> standwithukraine-1.2.5.tar.gz: standwithukraine/src/Service/StandWithUkraineFavicon.php <==
<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Elphin\IcoFileLoader\IcoFileService;

/**
 * Defines the favicon service.
 */
class StandWithUkraineFavicon implements StandWithUkraineFaviconInterface {

  use LoggerChannelTrait;

  /**
   * The flag colors.
   */
  const COLORS = [
    TRUE => [0x00, 0x5B, 0xBB],
    FALSE => [0xFF, 0xD5, 0x00],
  ];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface|null
   */
  protected $fileUrlGenerator;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    CacheBackendInterface $cache,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    FileUrlGeneratorInterface $file_url_generator = NULL
  ) {
    $this->cache = $cache;
    $this->configFactory = $config_factory;
    $this->setLoggerFactory($logger_factory);
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public function build($theme) {
    $cache = $this->cache->get('standwithukraine');

    if ($cache !== FALSE) {
      $items = $cache->data;

      if (isset($items[$theme])) {
        return $items;
      }
    }
    else {
      $items = [];
    }

    if (($filename = theme_get_setting('favicon.url', $theme)) === NULL) {
      return $items;
    }

    $loader = new IcoFileService();

    try {
      $icon = $loader->from(DRUPAL_ROOT . '/' . $filename);
    }
    catch (\Exception $exception) {
      $this->getLogger('standwithukraine')->warning($exception->getMessage());

      return $items;
    }

    $config = $this->configFactory->get('standwithukraine.settings');
    $settings = (array) $config->get('favicon.themes.' . $theme);
    $ratio = $settings['ratio'] ?? self::UNDEFINED_RATIO;

    /** @var \Elphin\IcoFileLoader\IconImage $image */
    foreach ($icon as $image) {
      $width = $image->width;
      $height = $image->height;
      $middle = $height / 100 * $ratio;
      $image = $loader->renderImage($image);
      $unique_alpha = $alpha_matrix = [];

      for ($vertical = $height - 1; $vertical >= 0; $vertical--) {
        for ($horizontal = 0; $horizontal < $width; $horizontal++) {
          $color_index = imagecolorat($image, $horizontal, $vertical);
          $rgba_color = imagecolorsforindex($image, $color_index);
          $alpha = $rgba_color['alpha'];
          $alpha_matrix[$horizontal][$vertical] = $alpha;

          if (empty($settings['background'])) {
            $unique_alpha[$vertical < $middle][$alpha] = 0;
          }
        }
      }

      for ($color_index = 0; $color_index < imagecolorstotal($image); $color_index++) {
        imagecolordeallocate($image, $color_index);
      }

      if (empty($settings['background'])) {
        foreach ($unique_alpha as $color_type => &$alpha_per_type) {
          foreach ($alpha_per_type as $alpha => &$color_index) {
            $color_index = call_user_func_array(
              'imagecolorallocatealpha',
              [$image, ...self::COLORS[$color_type], $alpha]
            );
          }

          unset($color_index);
        }
      }

      for ($vertical = $height - 1; $vertical >= 0; $vertical--) {
        for ($horizontal = 0; $horizontal < $width; $horizontal++) {
          $alpha = $alpha_matrix[$horizontal][$vertical];
          $color_type = $vertical < $middle;

          if (!empty($settings['background'])) {
            if (!$alpha) {
              continue;
            }

            $color_index = imagecolorat($image, $horizontal, $vertical);
            $rgba_color = imagecolorsforindex($image, $color_index);

            unset($rgba_color['alpha']);

            $alpha /= 128;

            $rgb_color = array_map(
              function ($background_color, $image_color) use ($alpha) {
                return round(
                  $image_color * (1 - $alpha) + $background_color * $alpha
                );
              },
              self::COLORS[$color_type],
              array_values($rgba_color)
            );

            $color_index = call_user_func_array(
              'imagecolorallocatealpha',
              [$image, ...$rgb_color, 0]
            );
          }
          else {
            $color_index = $unique_alpha[$color_type][$alpha];
          }

          imagesetpixel($image, $horizontal, $vertical, $color_index);
        }
      }

      $filename = sprintf(
        'public://standwithukraine-favicon-%s-%s.png',
        $theme,
        $dimensions = $height . 'x' . $width
      );

      imagepng($image, $filename);

      $items[$theme][$dimensions] = $this->fileUrlGenerator !== NULL
        ? $this->fileUrlGenerator->generateString($filename)
        : file_create_url($filename);
    }

    $this->cache->set('standwithukraine', $items);

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function isSupported($type) {
    return in_array($type, self::TYPES);
  }

}

==> standwithukraine-1.3.0.tar.gz: standwithukraine/src/Service/StandWithUkraineFavicon.php <==
<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Elphin\IcoFileLoader\IcoFileService;

/**
 * Defines the favicon service.
 */
class StandWithUkraineFavicon implements StandWithUkraineFaviconInterface {

  /**
   * The flag colors.
   */
  const COLORS = [
    TRUE => [0x00, 0x5B, 0xBB],
    FALSE => [0xFF, 0xD5, 0x00],
  ];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface|null
   */
  protected $fileUrlGenerator;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    CacheBackendInterface $cache,
    ConfigFactoryInterface $config_factory,
    FileUrlGeneratorInterface $file_url_generator = NULL
  ) {
    $this->cache = $cache;
    $this->configFactory = $config_factory;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public function build($theme) {
    $cache = $this->cache->get('standwithukraine');

    if ($cache !== FALSE) {
      $items = $cache->data;

      if (isset($items[$theme])) {
        return $items;
      }
    }
    else {
      $items = [];
    }

    if (($filename = theme_get_setting('favicon.url', $theme)) === NULL) {
      return $items;
    }

    $loader = new IcoFileService();

    try {
      $icon = $loader->from(DRUPAL_ROOT . '/' . $filename);
    }
    catch (\Exception $exception) {
      return $items;
    }

    $config = $this->configFactory->get('standwithukraine.settings');
    $ratio = $config->get('favicon.ratios.' . $theme) ?? self::UNDEFINED_RATIO;

    /** @var \Elphin\IcoFileLoader\IconImage $image */
    foreach ($icon as $image) {
      $width = $image->width;
      $height = $image->height;
      $middle = $height / 100 * $ratio;
      $image = $loader->renderImage($image);
      $unique_alpha = $alpha_matrix = [];

      for ($vertical = $height - 1; $vertical >= 0; $vertical--) {
        for ($horizontal = 0; $horizontal < $width; $horizontal++) {
          $color_index = imagecolorat($image, $horizontal, $vertical);
          $rgba_color = imagecolorsforindex($image, $color_index);
          $alpha = $rgba_color['alpha'];
          $unique_alpha[$vertical < $middle][$alpha] = 0;
          $alpha_matrix[$horizontal][$vertical] = $alpha;
        }
      }

      for ($color_index = 0; $color_index < imagecolorstotal($image); $color_index++) {
        imagecolordeallocate($image, $color_index);
      }

      foreach ($unique_alpha as $color_type => &$alpha_per_type) {
        foreach ($alpha_per_type as $alpha => &$color_index) {
          $color_index = call_user_func_array(
            'imagecolorallocatealpha',
            array_merge([$image], self::COLORS[$color_type], [$alpha])
          );
        }

        unset($color_index);
      }

      for ($vertical = $height - 1; $vertical >= 0; $vertical--) {
        for ($horizontal = 0; $horizontal < $width; $horizontal++) {
          $alpha = $alpha_matrix[$horizontal][$vertical];

          imagesetpixel(
            $image,
            $horizontal,
            $vertical,
            $unique_alpha[$vertical < $middle][$alpha]
          );
        }
      }

      $filename = sprintf(
        'public://standwithukraine-favicon-%s-%s.png',
        $theme,
        $dimensions = $width . 'x' . $height
      );

      imagepng($image, $filename);

      $items[$theme][$dimensions] = $this->fileUrlGenerator !== NULL
        ? $this->fileUrlGenerator->generateString($filename)
        : file_create_url($filename);
    }

    $this->cache->set('standwithukraine', $items);

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function isSupported($type) {
    return substr($type, 0, 6) === 'image/' &&
      in_array(substr($type, 6), ['png', 'vnd.microsoft.icon', 'x-icon']);
  }

}
